#    MIDAS Access Server  common procedures 


global MIDAS_RegisterAccessServer_Verbose      ;# for some reason this does not create a global variable
set    MIDAS_RegisterAccessServer_Verbose 0


    proc InquireRegisters {Pattern} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterIndex RegisterIndex
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::InquireRegisters with parameter $Pattern"}
  
        set names [array names RegisterNames -glob $Pattern]

        return [rpcvar::rpcvar string "0 $names"]
    }

    proc DeleteRegister {Register} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::DeleteRegister with parameter $Register"}

        if {![info exists RegisterNames($Register)]} then {
             return [rpcvar::rpcvar string "0x10004 Register name does not exist"]
        }

        set Index $RegisterNames($Register)
        upvar R$Index R
        array unset R

        unset RegisterNames($Register)

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc DeleteRegisters {Pattern} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::DeleteRegisters with parameter $Pattern"}

        foreach Register [array names RegisterNames -glob $Pattern] {
            set Index $RegisterNames($Register)
            upvar R$Index R
            array unset R

            unset RegisterNames($Register)
        }

        return [rpcvar::rpcvar string "0 OK"]
    }

    proc WriteRegisterAttributes {Register Attributes} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::WriteRegisterAttributes with parameters $Register $Attributes"}

        if {![info exists RegisterNames($Register)]} then {
             return [rpcvar::rpcvar string "0x10004 Register name does not exist"]
        }

        set response [do_WriteRegisterAttributes $Register $Attributes]
 
        return [rpcvar::rpcvar string "$response"]
    }


    proc WriteRegistersAttributes {Pattern Attributes} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::WriteRegistersAttributes with parameters $Pattern $Attributes"}

        foreach Register [array names RegisterNames -glob $Pattern] {

            do_WriteRegisterAttributes $Register $Attributes
        }
  
        return [rpcvar::rpcvar string "0 OK"]
    }

    proc ReadRegisterAttributes {Register} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::ReadRegisterAttributes with parameter $Register"}

        if {![info exists RegisterNames($Register)]} then {
             return [rpcvar::rpcvar string "0x10004 Register name does not exist"]
        }

        set attributes [do_ReadRegisterAttributes $Register]

        return [rpcvar::rpcvar string "$attributes"]
    }

    proc ReadRegistersAttributes {Pattern} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::ReadRegisterAttributes with parameter $Pattern"}

        set Response {}

        foreach Register [array names RegisterNames -glob $Pattern] {

            set response [do_ReadRegisterAttributes $Register]
            if {[head $response] == 0} {lappend Response [list $Register [tail $response]]}
        }

        return [rpcvar::rpcvar string "0 $Response"]
    }

    proc WriteRegister {Register Value} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::WriteRegister with parameters $Register $Value"}

        if {![info exists RegisterNames($Register)]} then {
             return [rpcvar::rpcvar string "0x10004 Register name does not exist"]
        }

        set response [do_WriteRegister $Register $Value]

        return [rpcvar::rpcvar string "$response"]
    }

    proc WriteRegisters {Pattern Value} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::WriteRegisters with parameters $Pattern $Value"}


        foreach Register [array names RegisterNames -glob $Pattern] {

            do_WriteRegister $Register $Value
        }
  
        return [rpcvar::rpcvar string "0 OK"]
    }

    proc InitialiseRegister {Register} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::InitialiseRegister with parameter $Register"}

        if {![info exists RegisterNames($Register)]} then {
             return [rpcvar::rpcvar string "0x10004 Register name does not exist"]
        }

        set Index $RegisterNames($Register)
        upvar R$Index R

        if {![info exists R(i)]}  then {
             return [rpcvar::rpcvar string "0x2004 Register cannot be initialised"]
        }

        if {$R(i) == 0xa55a}  then {
             return [rpcvar::rpcvar string "0x2004 Register has no initial value"]
        }

        set response [do_WriteRegister $Register $R(i)]

        return [rpcvar::rpcvar string "$response"]
    }

    proc InitialiseRegisters {Pattern} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::InitialiseRegisters with parameter $Pattern"}

        foreach Register [array names RegisterNames -glob $Pattern] {

            set Index $RegisterNames($Register)
            upvar R$Index R

            if {![info exists R(i)]}  then {continue}
            if {$R(i) == 0xa55a}  then {continue}

            do_WriteRegister $Register $R(i)
        }

        return [rpcvar::rpcvar string "0 OK"]
    }


    proc ReadRegister {Register} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::ReadRegister with parameter $Register"}

        if {![info exists RegisterNames($Register)]} then {
             return [rpcvar::rpcvar string "0x10004 Register name does not exist"]
        }

        set response [do_ReadRegister $Register]

        return [rpcvar::rpcvar string "$response"]
    }


    proc ReadRegisters {Pattern} {

        global MIDAS_RegisterAccessServer_Verbose
        upvar RegisterNames RegisterNames

        if {$MIDAS_RegisterAccessServer_Verbose} \
             {puts "called method RegisterAccessServer::ReadRegisters with parameter $Pattern"}

        set Response {}

        foreach Register [array names RegisterNames -glob $Pattern] {

            set response [do_ReadRegister $Register]
            if {[head $response] == 0}  {lappend Response [list $Register [tail $response]]}
        }
 
        return [rpcvar::rpcvar string "0 $Response"]
    }

